#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpgraphic16.h>

#include "gpmain.h"
#include "../include/global.h"
#include "../include/clock.h"
#include "../include/graphics16.h"
#include "../include/okf.h"
#include "../include/fileSelector.h"

#include "project.h"
#include "fonts.h"
#include "intro.h"
#include "toolbox.h"
#include "properties.h"


int mbShowFreeMem = FALSE;

static void Initialize(void)
{
  InitGraphics(16);
  GpFatInit();

  // Initialize alpha blending look up tables
  InitializeAlphaLUT(ALT_4LEVELS,  gtAlpha4);
  InitializeAlphaLUT(ALT_8LEVELS,  gtAlpha8);
  InitializeAlphaLUT(ALT_16LEVELS, gtAlpha16);
  InitializeAlphaLUT(ALT_32LEVELS, gtAlpha32);

  OkfInitialize(gtSurface, &giSurface);

  // OkfInitialize initializes a default 32 bit alpha LUT
  // In this program we'll use our own alpha LUT
  OkfInitializeAlphaLUT(giAlphaLevels, gtAlphaLUT);

  OkfRegister(verdana8b_pm_okf);
  OkfRegister(verdana10_okf);
  OkfRegister(verdana8_okf);
  OkfRegister(courierNew9_okf);
  OkfRegister(gp32Buttons16_p_okf);
  OkfClone(2);
  OkfApplyEffect(okf.pFonts->lastHandle, okfAddColor, RGB(31, 15, 0), 0);
}

static void Terminate(void)
{
  OkfTerminate();
}

//***************************************


static void Help(int option)
{
  unsigned char *text;

  switch (option)
  {
    case 0:
      text = "Welcome to the OKF font tester program and its help system. At any time in the program you can press SELECT to get help about a particular function.\n\n"
             "But first of all, what are OKF fonts?\n"
             "Well, OKF stands for Oankali Font. Not very original I know :-)\n\n"
             "OKF fonts are a new font format specially designed to suit the needs of programmers of Game Park GP32 handheld video system.\n"
             "With these fonts and the OKF Font Engine, programmers have the possibility to rapidly prepare menus, options screens, etc... of the best quality.\n\n"
             "OKF fonts are fonts of proportional width that can be antialiased or shadowed via a mask. It's the responsibility of the designer of the font to include the correct mask to achieve the right effect.\n"
             "Finally, bitmaps in OKF fonts (font and/or mask bitmaps) can be run length encoded to save space on your SMC.\n\n"
             "On the other side, the OKF Font Engine has been designed to make programmers life easier, providing some high level functions to print multiline texts, with 3 types of justification, real time effects like opacity, color overlaying and even shadow positioning.\n"
             "The font engine can load fonts from the SMC card or register them directly from the memory, embedded in the program.\n\n"
             "And how can I create an OKF font?\n"
             "For the moment, there is only one way to do it, using GPFontMaker v2.0.\n"
             "Select a True Type font, set properties, export bitmaps (font and/or mask), post process them with your preferred image processor and finally generate the OKF font file.\n"
             "The font is ready to be used with the OKF Font Engine.\n\n"
             "Use OKF Font Tester to test your fonts or play with the fonts of other designers.";
      break;

    case 1:
      text = "You can change the current font with this option.\n\n"
             "The first number represents its place in the font list, not its handle.\n"
             "The second number is the total of fonts registered at the moment.\n\n"
             "When executing OKF Font Tester, 5 embedded fonts are registered automatically (1 to 5) and another one is cloned from font number 2. A permanent color change is then applied to this last font.\n"
             "That's why it's not possible to release the first 6 fonts as there are considered system fonts.";
      break;

    case 2:
      text = "Enter the Font Toolbox to test your registered fonts.";
      break;

    case 3:
      text = "With this option you can browse all the properties of your registered fonts.";
      break;

    case 4:
      text = "There is 3 more embedded fonts in OKF Font Tester for you to play with.\n\n"
             "You can register how many fonts as you want, well, in reality while you have enough memory available.\n\n"
             "If you register more than 3 fonts, the remaining fonts will be duplicated.";
      break;

    case 5:
      text = "You have designed an OKF font with GPFontMaker?\n"
             "Load it and register it with this option.\n"
             "You will then be able to test it as the other fonts.\n";
      break;

    case 6:
      text = "With this option you can clone a font.\n\n"
             "Not useful you say?\n"
             "It is, if you want to work on a copy of one of the 6 system fonts provided in this test program and apply permanent changes on it.\n"
             "(See: Font Toolbox/Advanced/Apply changes)";
      break;

    case 7:
      text = "Release a font with this option.\n"
             "You will recover allocated memory.\n\n"
             "Fonts 1 to 6 can't be released in this test program, they are used as system fonts.";
      break;

    case 8:
      text = "Use this option, to show or hide the available memory.";
      break;

    case 9:
      text = "You are tired to play with OKF Font Tester and you want to execute another application on your GP32?\n"
             "This is your option.";
      break;

    default:
      return;
    }

  // Show help
  MessageBox("Main Menu Help", 30, 25, 260, 170, text);
}

static void RefreshScreen(int currentOption)
{
  OKFENGINE oldState;
  char *msg;
  char str[20];

  // Save current state
  oldState = okf;

  // Init new state
  OkfSetToDefaults();
  okf.font = 1;
  okf.lineSeparation = -1;
  okf.effect.color = RGB(31, 15, 0);
  okf.effect.intensity = 15;

  // Background
  GpRectFill16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT, RGB(31, 31, 31));

  // Title
  okf.effect.type = okfOverlay;
  okf.justify = okfCenter;
  OkfPrintAt(GPC_LCD_WIDTH / 2, 10, "OKF FONT TESTER");
  okf.justify = okfJustifyLeft;
  okf.effect.type = okfNone;

  // Menu options
  okf.effect.type = (currentOption == 1) ? okfAddColor : okfNone;
  OkfPrintAt(20, 35, "Current font");

  okf.effect.type = okfAddColor;
  okf.effect.color = RGB(0, 0, 20);
  gm_sprintf(str, "%ld of %ld\n", ListGetIndexByHandle(okf.pFonts, oldState.font) + 1, okf.pFonts->count);
  OkfPrintAt(130, 35, str);
  okf.effect.color = RGB(31, 15, 0);

  okf.effect.type = (currentOption == 2) ? okfAddColor : okfNone;
  OkfPrintAt(20, okf.y + 5, "Font toolbox\n");

  okf.effect.type = (currentOption == 3) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, "Display font properties\n");

  okf.effect.type = (currentOption == 4) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, "Register embedded OKF Font\n");

  okf.effect.type = (currentOption == 5) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, "Load & Register OKF Font\n");

  okf.effect.type = (currentOption == 6) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, "Clone current font\n");

  okf.effect.type = (currentOption == 7) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, "Release current font\n");

  if (mbShowFreeMem)
    msg = "Hide free memory\n";
  else
    msg = "Show free memory\n";
  okf.effect.type = (currentOption == 8) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y, msg);

  okf.effect.type = (currentOption == 9) ? okfAddColor : okfNone;
  OkfPrintAt(okf.x, okf.y + 5, "Exit");

  // Status message
  GpRectFill16(NULL, &gtSurface[giSurface], 0, 190, GPC_LCD_WIDTH, 1, RGB(31, 15, 0));
  switch (oldState.lastError)
  {
  case okfOk:                  msg = "Last action executed correctly."; break;
  case okfNotInitialized:      msg = "OKF font engine not initialized."; break;
  case okfMemory:              msg = "Out of memory."; break;
  case okfNoCurrentFont:       msg = "No current font selected."; break;
  case okfHandleNotValid:      msg = "The handle of the font is not valid."; break;
  case okfFontFileNotFound:    msg = "Font file not found."; break;
  case okfBadFontFile:         msg = "The file is not a font or is corrupted."; break;
  case okfVersionNotSupported: msg = "The version of the file is not supported."; break;
  case okfSMCError:            msg = "SMC read error."; break;
  case 100:                    msg = "Fonts 1 to 6 are system fonts in this program."; break;

  case 200:                    msg = "Press SELECT for help."; break;
  case 201:                    msg = "Change current font."; break;
  case 202:                    msg = "To play with registered fonts."; break;
  case 203:                    msg = "Displays properties of registered fonts."; break;
  case 204:                    msg = "Registers a font embedded in the FXE."; break;
  case 205:                    msg = "Loads and registers a font from the SMC card."; break;
  case 206:                    msg = "Makes a complete clone of the current font."; break;
  case 207:                    msg = "Releases the current font from memory."; break;
  case 208:                    msg = "Hides or shows actual free memory."; break;
  case 209:                    msg = "Reboots GP32."; break;
  default:                     msg = "";
  }
  okf.font = 6;
  okf.effect.type = okfNone;
  OkfPrintAt(10, 195, msg);

  // Copyright
  GpRectFill16(NULL, &gtSurface[giSurface], 0, 215, GPC_LCD_WIDTH, 1, RGB(31, 15, 0));
  OkfSetToDefaults();
  okf.font = 1;
  OkfPrintAt(12, 220, AUTHOR);
  OkfPrintAt(170, 220, COPYRIGHT);

  // Available memory
  if (mbShowFreeMem)
  {
    okf.font = 3;
    OkfPrintLongAt(2, 0, gm_availablesize(), "%ld");
  }

  SurfaceFlip();

  // Restore state
  okf = oldState;
}

static void MainMenu(void)
{
  FILESELECTOR fileSelector;
  int tick;
  int pressedKeys, newKeys, oldKeys;
  int currentOption=0;
  int embeddedFonts=0;
  int handle;

  // Customize file selector
  FileSelectorInit(&fileSelector);
  gm_strcpy(fileSelector.currentPath, "gp:\\gpmm");
  fileSelector.pFilters = "*.okf";
  fileSelector.pathFont = 2;
  fileSelector.folderFont = 2;
  fileSelector.fileFont = 6;
  fileSelector.sizesFont = 6;
  fileSelector.totalsFont = 2;
  fileSelector.backColor = RGB(31, 31, 31);
  fileSelector.scrollBarColor = RGB(31, 15, 0);
  fileSelector.selectionColor = RGB(31, 15, 0);
  fileSelector.selectionIntensity = 8;
  fileSelector.separationLinesColor = RGB(31, 15, 0);
  fileSelector.repetitionDelay = 600;
  fileSelector.repetitionSpeed = 100;

  okf.lastError = 200;

  while(TRUE)
  {
    tick = GpTickCountGet();
    
    // Refresh screen
    RefreshScreen(currentOption);

    // Get buttons state
    oldKeys = pressedKeys;
    newKeys = pressedKeys = GpKeyGet();

    // Filter buttons
    if ((newKeys & GPC_VK_UP) && (oldKeys & GPC_VK_UP))
      newKeys = newKeys & ~GPC_VK_UP;
    if ((newKeys & GPC_VK_DOWN) && (oldKeys & GPC_VK_DOWN))
      newKeys = newKeys & ~GPC_VK_DOWN;
    if ((newKeys & GPC_VK_FA) && (oldKeys & GPC_VK_FA))
      newKeys = newKeys & ~GPC_VK_FA;
    if ((newKeys & GPC_VK_FB) && (oldKeys & GPC_VK_FB))
      newKeys = newKeys & ~GPC_VK_FB;

    // Manage buttons
    if (newKeys & GPC_VK_SELECT)
    {
      Help(currentOption);
    }
    else if (newKeys & GPC_VK_UP)
    {
      if (currentOption > 1)
        currentOption--;
      else
        currentOption = 9;
      okf.lastError = 200 + currentOption;
    }
    else if (newKeys & GPC_VK_DOWN)
    {
      if (currentOption < 9)
        currentOption++;
      else
        currentOption = 1;
      okf.lastError = 200 + currentOption;
    }
    else if (newKeys & GPC_VK_FA)
    {
      switch (currentOption)
      {
        case 1: // Change current font
          if (okf.pFonts->count > 0)
          {
            LISTITEM *pItem;

            if ((pItem = ListGetItemByHandle(okf.pFonts, okf.font)->pNext) != NULL)
              okf.font = pItem->handle;
            else
              okf.font = okf.pFonts->pFirst->handle;
          }
          else
            okf.lastError = okfOk;
          break;

        case 2: // Font toolbox
          if (okf.font > 0)
            Toolbox();
          else
            okf.lastError = okfNoCurrentFont;
          break;

        case 3: // Display font properties
          if (okf.font > 0)
            Properties();
          else
            okf.lastError = okfNoCurrentFont;
          break;

        case 4: // Register embedded OKF Font
          if (++embeddedFonts > 4)
            embeddedFonts = 1;
          switch (embeddedFonts)
          {
          case 1: OkfRegister(comicSansMS12b_p_okf); break;
          case 2: OkfRegister(comicSansMS12b_pm_okf); break;
          case 3: OkfRegister(arialBlack12b_pm_okf); break;
          case 4: OkfRegister(yakapGold16b_pm_okf); break;
          }
          okf.font = okf.pFonts->pLast->handle;
          break;

        case 5: // Load & Register OKF Font
          if (FileSelector(&fileSelector) == 1)
          {
            OkfLoadAndRegister(fileSelector.filename);
            okf.font = okf.pFonts->pLast->handle;
          }
          break;

        case 6: // Clone current font
          OkfClone(okf.font);
          okf.font = okf.pFonts->pLast->handle;
          break;

        case 7: // Release current font
          if (okf.font == 0)
            okf.lastError = okfNoCurrentFont;
          else if (okf.font <= 6)
            okf.lastError = 100;
          else
          {
            handle = okf.font;
            if (ListGetItemByHandle(okf.pFonts, handle) == okf.pFonts->pLast)
              okf.font = okf.pFonts->pLast->pPrevious->handle;
            else
              okf.font = ListGetItemByHandle(okf.pFonts, handle)->pNext->handle;
            OkfRelease(handle);
          }
          break;

        case 8: // Show/Hide free memory
          mbShowFreeMem = !mbShowFreeMem;
          break;

        case 9: // Exit
          return;
      }
    }
    else if (newKeys & GPC_VK_FB)
    {
      switch (currentOption)
      {
        case 1: // Change current font
          if (okf.pFonts->count > 0)
          {
            LISTITEM *pItem;

            if ((pItem = ListGetItemByHandle(okf.pFonts, okf.font)->pPrevious) != NULL)
              okf.font = pItem->handle;
            else
              okf.font = okf.pFonts->pLast->handle;
          }
          else
            okf.lastError = okfOk;
          break;
      }
    }
    
    while ((GpTickCountGet() - tick) < 50);
  }
}


void GpMain (void * arg)
{
  Initialize();
  Intro();
  MainMenu();
  Terminate();
  GpAppExit();
}
