#include <gpdef.h>
#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpgraphic16.h>

#include "../include/global.h"
#include "../include/clock.h"
#include "../include/graphics16.h"
#include "../include/okf.h"

#include "project.h"
#include "fonts.h"
#include "bitmaps.h"


void Intro(void)
{
  unsigned short *pBg;
  int handle;
  int i, canceled=FALSE;

  SetClockSpeed(CLOCK40);

  // Register new font
  handle = OkfRegister(comicSansMS12b_pm_okf);

  // Set OKF Font Engine
  OkfSetToDefaults();

  // Step 1: Avatar
  if (!canceled)
  {
    //- Background
    GpRectFill16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT, RGB(31, 31, 31));
    GpBitBlt16(NULL, &gtSurface[giSurface], 30, 50, avatar_width, avatar_height,
               (unsigned char *) avatar_bitmap, 0, 0, avatar_width, avatar_height);

    SurfaceFlip();

    canceled = WaitKey(500);
  }
  pBg = SaveScreen();

  // Step 2: Title text
  if (!canceled)
  {
    okf.font = handle;
    okf.shadow.x = 8;
    okf.shadow.y = 4;
    okf.shadow.opacity = 0;
    okf.charSeparation = 46;
    for (i = 320; !canceled && (i >= 140); i-=2)
    {
      //- Background
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
                 (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);

      //- Text
      if (i % 4 == 0)
        okf.charSeparation--;
      OkfPrintAt(i, 70, "OKF Font Tester");

      SurfaceFlip();

      if (GpKeyGet())
        canceled = TRUE;
    }
  }

  // Step 3: Version
  if (!canceled)
  {
    //- Background
    GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
               (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);
    //- Text
    OkfPrintAt(140, 70, "OKF Font Tester");
    okf.font = 3;
    OkfPrintAt(140, 95, "v2.0");
    okf.font = handle;

    SurfaceFlip();

    ReleaseScreen(pBg);
    pBg = SaveScreen();
  }

  // Step 4: Title shadow (opacity)
  if (!canceled)
  {
    while (!canceled && (okf.shadow.opacity <= 31))
    {
      //- Background
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
                 (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);

      //- Text
      OkfPrintAt(140, 70, "OKF Font Tester");

      SurfaceFlip();

      okf.shadow.opacity++;

      canceled = WaitKey(30);
    }
    if (!canceled)
      canceled = WaitKey(200);
  }

  // Step 5: Title shadow (move)
  if (!canceled)
  {
    while (!canceled && (okf.shadow.x >= 0))
    {
      //- Background
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
                 (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);

      //- Text
      OkfPrintAt(140, 70, "OKF Font Tester");

      SurfaceFlip();

      okf.shadow.x--;
      okf.shadow.y = okf.shadow.x / 2;

      canceled = WaitKey(60);
    }
    ReleaseScreen(pBg);
    pBg = SaveScreen();
  }

  // Step 6: Author info (opacity)
  if (!canceled)
  {
    okf.opacity = 0;
    while (!canceled && (okf.opacity <= 31))
    {
      //- Background
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
                 (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);

      //- Text
      okf.font = 1;
      okf.charSeparation = -1;
      OkfPrintAt(30, 150, AUTHOR);
      OkfPrintAt(170, 150, COPYRIGHT);
      okf.charSeparation = 0;
      okf.font = 2;
      OkfPrintAt(30, 170, EMAIL);

      SurfaceFlip();

      okf.opacity++;

      canceled = WaitKey(40);
    }
    ReleaseScreen(pBg);
    pBg = SaveScreen();
  }

  // Step 7: Email (Overlay)
  if (!canceled)
  {
    okf.font = 2;
    okf.effect.type = okfOverlay;
    okf.effect.color = RGB(31, 15, 0);
    okf.effect.intensity = 0;
    while (!canceled && (okf.effect.intensity <= 31))
    {
      //- Background
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
                 (unsigned char *) pBg, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT);

      //- Text
      OkfPrintAt(30, 170, EMAIL);

      SurfaceFlip();

      okf.effect.intensity++;

      canceled = WaitKey(25);
    }
  }

  // Release allocated ressources
  ReleaseScreen(pBg);
  OkfRelease(handle);
  OkfSetToDefaults();

  if (!canceled)
    WaitKey(2000);
  while (GpKeyGet());

  SetClockSpeed(CLOCK80);

  FadeOut();
  Wait(1000);
}
