#include <gpdef.h>
#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpgraphic16.h>

#include "../include/global.h"
#include "../include/graphics16.h"
#include "../include/okf.h"

#include "project.h"
#include "toolbox.h"
#include "bitmaps.h"

#define MENUUP      0x01
#define MENUDOWN    0x02
#define MENULEFT    0x04
#define MENURIGHT   0x08
#define MENUFA      0x10
#define MENUFB      0x20


static void CustomBackground(int x, int y, int width, int height)
{
  int i;

  // We imitate OKF Engine background creation
  OverlayColor16(&gtSurface[giSurface], x, y, width, height,
                 okf.background.color, okf.background.opacity);
  
  // We paint a border
  for (i=x + 5 - 2; i < x + width - 5 + 2; i+=5)
  {
    TransFade16(NULL, &gtSurface[giSurface], i, y - 2, 5, 5,
                (unsigned char *) border_bitmap, 6, 0, border_width, border_height,
                border_transColor, okf.background.opacity);
    TransFade16(NULL, &gtSurface[giSurface], i, y + height - 5 + 2, 5, 5,
                 (unsigned char *) border_bitmap, 6, 11, border_width, border_height,
                 border_transColor, okf.background.opacity);
  }

  for (i=y + 5 - 2; i < y + height - 5 + 2; i+=5)
  {
    TransFade16(NULL, &gtSurface[giSurface], x - 2, i, 5, 5,
                (unsigned char *) border_bitmap, 0, 6, border_width, border_height,
                border_transColor, okf.background.opacity);
    TransFade16(NULL, &gtSurface[giSurface], x + width - 5 + 2, i, 5, 5,
                (unsigned char *) border_bitmap, 11, 6, border_width, border_height,
                border_transColor, okf.background.opacity);
  }

  TransFade16(NULL, &gtSurface[giSurface], x - 2, y - 2, 5, 5,
              (unsigned char *) border_bitmap, 0, 0, border_width, border_height,
              border_transColor, okf.background.opacity);
  TransFade16(NULL, &gtSurface[giSurface], x + width - 5 + 2, y - 2, 5, 5,
              (unsigned char *) border_bitmap, 11, 0, border_width, border_height,
              border_transColor, okf.background.opacity);
  TransFade16(NULL, &gtSurface[giSurface], x + width - 5 + 2, y + height - 5 + 2, 5, 5,
              (unsigned char *) border_bitmap, 11, 11, border_width, border_height,
              border_transColor, okf.background.opacity);
  TransFade16(NULL, &gtSurface[giSurface], x - 2, y + height - 5 + 2, 5, 5,
              (unsigned char *) border_bitmap, 0, 11, border_width, border_height,
              border_transColor, okf.background.opacity);
}

static void Help(int option)
{
  unsigned char *text;

  switch (option)
  {
    case 0:
      text = "The Font Toolbox is designed to help you testing OKF fonts.\n\n"
             "The hierarchical menu is very easy to use:\n\n"
             "- Active navigation buttons are always displayed at the bottom right of the screen.\n\n"
             "- Press UP and DOWN on your joypad to navigate through options of a submenu.\n\n"
             "- Press LEFT on your joypad to go back one level.\n\n"
             "- Press RIGHT on your joypad to go down one level if your are on an submenu option.\n\n"
             "- Press A/B to change the value of an option.\n\n"
             "- Press LEFT/RIGHT shoulders at any time to scroll the sample text up and down.\n\n"
             "- Press SELECT to get the contextual help of an option.\n\n"
             "- Press START to exit to the Main Menu.";
      break;

    case 100000:
      text = "You can change the current font with this option.\n"
             "All settings will remain active.";
      break;

    case 200000:
      text = "To help you in testing your OKF font you have the possibility the change the background of the screen.\n\n"
             "You can choose between a bitmap or a plain color.";
      break;
    case 210000:
      text = "Choose the type of screen background you want to use.\n\n"
             "You can choose between the default bitmap or a colored background.";
      break;
    case 220000:
    case 322000:
    case 422200:
    case 530000:
      text = "You can set the color by choosing a predefined one, or by setting its RGB components individually.";
      break;
    case 221000:
    case 322100:
    case 422210:
    case 531000:
      text = "Choose a predefined color.";
      break;
    case 221100:
    case 322110:
    case 422211:
    case 531100:
      text = "Choose black color.";
      break;
    case 221200:
    case 322120:
    case 422212:
    case 531200:
      text = "Choose white color.";
      break;
    case 221300:
    case 322130:
    case 422213:
    case 531300:
      text = "Choose grey color.";
      break;
    case 221400:
    case 322140:
    case 422214:
    case 531400:
      text = "Choose red color.";
      break;
    case 221500:
    case 322150:
    case 422215:
    case 531500:
      text = "Choose green color.";
      break;
    case 221600:
    case 322160:
    case 422216:
    case 531600:
      text = "Choose blue color.";
      break;
    case 222000:
    case 322200:
    case 422220:
    case 532000:
      text = "Set the red component of the color.";
      break;
    case 223000:
    case 322300:
    case 422230:
    case 533000:
      text = "Set the green component of the color.";
      break;
    case 224000:
    case 322400:
    case 422240:
    case 534000:
      text = "Set the blue component of the color.";
      break;

    case 300000:
      text = "An OKF font has predefined properties settled by the designer of the font.\n"
             "However the OKF Font Engine allows you to apply real time changes over the font to customize it to your needs.\n\n"
             "For example, you can change the color of the font by applying a color effect, or change the default char spacing or the default line spacing.\n\n"
             "You can even change the color of the shadow if there is one, or its position in reference to the real character.";
      break;
    case 310000:
      text = "Changing the opacity value, you can make a text more or less visible on the screen, as if it was transparent.\n\n"
             "Range: 0 (invisible) - 31 (opaque)";
      break;
    case 320000:
      text = "Some special effects can be applied on a font in real time:\n\n"
             "- Color overlying\n"
             "- Color addition\n"
             "- Color subtraction\n"
             "- Gray scaling\n\n"
             "Only one special effect can be applied at a time.";
      break;
    case 321000:
    case 422100:
      text = "Choose the type of the special effect you want to apply.\n\n"
             "You can choose between Overlay, AddColor or SubtractColor.";
      break;
    case 323000:
    case 422300:
      text = "Changing the intensity value, you can make the Overlay effect more or less visible.\n\n"
             "Range: 0 (no effect) - 31 (maximum effect)";
      break;
    case 330000:
      text = "The OKF Font Engine can justify your text in 3 ways:\n\n"
             "- Left justification\n"
             "- Centered\n"
             "- Right justification\n";
      break;
    case 340000:
      text = "Every OKF font has a default char separation settled by the designer of the font.\n"
             "To this default char separation you can add a value to change the aspect of the font.\n\n"
             "Range: -128 to +127 pixels";
      break;
    case 350000:
      text = "Every OKF font has a default line separation settled by the designer of the font.\n"
             "To this default line separation you can add a value to change the line spacing of the font.\n\n"
             "Range: -128 to +127 pixels";
      break;

    case 400000:
      text = "An OKF font can have one mask.\n\n"
             "The mask can be of two types: Generic or Shadow.\n\n"
             "The type is defined by the designer of the font and should not be changed by your program.\n\n"
             "Generic masks can be used for many fancy effects, but it is mainly used for antialising.\n"
             "Shadow masks are what there are, just a mask to display a shadow underneath the character.\n\n"
             "Only one mask can be included in an OKF font. That doesn't mean you can't have a font antialiased and with a shadow.\n"
             "If you want to create a font antialiased and with a shadow you will have to use a Generic mask that will include all the necessary information.\n"
             "In these cases, the shadow can't be positioned.";
      break;
    case 410000:
      text = "The mask of an OKF font can be deactivated by program.\n\n"
             "Choose if you want it to be active or not.";
      break;
    case 420000:
      text = "Shadow masks are a special type of OKF font masks with some additional properties to change its color, its opacity or its position in reference to the real characters.";
      break;
    case 421000:
      text = "Changing the opacity value, you can make a shadow more or less visible on the screen.\n\n"
             "Range: 0 (invisible) - 31 (opaque)";
      break;
    case 422000:
      text = "Some special effects can be applied on a shadow in real time:\n\n"
             "- Color overlaying\n"
             "- Color addition\n"
             "- Color subtraction\n"
             "- Gray scaling\n\n"
             "Only one special effect can be applied at a time.";
      break;
    case 423000:
      text = "Press A to start positionning manually the shadow.";
      break;

    case 500000:
      text = "You can make the OKF Font Engine paint a background automatically behind the text.\n\n"
             "These feature is not really binded to the font but it can be useful to select a word, a text or a menu option.\n\n"
             "There are to types of background, one painted automatically by the OKF Font Engine, and another painted by a callback function provided by the programmer.\n\n"
             "A normal background has a color, an opacity, and a size relative to the size of the text to be displayed in the foreground.";
      break;
    case 510000:
      text = "Changing the opacity value, you can make a background more or less visible on the screen, as if it was transparent.\n\n"
             "Range: 0 (invisible) - 31 (opaque)";
      break;
    case 520000:
      text = "Choose the type of background you want to use.\n\n"
             "You can choose between the default colored background painted by the OKF Font Engine or a custom background painted by a callback function provided by the programmer.";
      break;
    case 540000:
      text = "Press A to start positionning manually the background.";
      break;
    case 550000:
      text = "Press A to start resizing manually the background.";
      break;

    case 600000:
      text = "Some advanced features can be achieved through the OKF Font Engine like area clipping, or applying permanent effects.";
      break;
    case 610000:
      text = "Through the 'clip' structure of the OKF Font Engine you can delimit the area where the printing functions will really print, even if you print outside the delimited area.\n\n"
             "Press A to start moving a clipping area and see the results.";
      break;
    case 620000:
      text = "The OKF Font Engine allows you to apply permanent changes on an OKF font.\n\n"
             "As an example: clone a font, apply some font effects (Overlay, AddColor or SubtractColor), change properties (Char separation, Line separation) and finally apply permanently the changes.\n"
             "After that, even if you leave the Font Toolbox, you'll we see that the effects remain active in the font clone.\n\n"
             "Just for fun, try it on the system fonts, but be careful!! :-)";
      break;
    case 630000:
      text = "The OKF Font Engine has a prebuild alpha blending look up table to speed up the rendering of the fonts.\n"
             "The default precision of the LUT when initializing the engine is 32 levels.\n"
             "This precision can be changed at any time, or you can even use a LUT of another library provided it has the same structure.\n\n"
             "Changing the precision of the alpha blending look up table will modify the quality of the rendering of the fonts when using masks or the opacity attribute.\n\n"
             "Possible values:\n"
             "- 32 levels\n"
             "- 16 levels\n"
             "- 8  levels\n"
             "- 4  levels\n";
      break;
  
    default:
      return;
    }

  // Show help
  MessageBox("ToolBox Help", 30, 25, 260, 170, text);
}

void Toolbox(void)
{
  LISTITEM *pItem;
  OKFENGINE oldState;
  OKFFONT *pFont;
  OKFEFFECT *pShadowE;
  int pressedKeys, newKeys, oldKeys;
  int keyTick=0;
  int menuLevel=0, menuKeys=0, curOption0=0, curOption1=0, curOption2=0,
                               curOption3=0, curOption4=0, curOption5=0;
  int showScreenBackground=TRUE, moveShadow=FALSE, clipFont=FALSE,
      moveBackground=FALSE, resizeBackground=FALSE;
  unsigned short bgColor=RGB(31, 31, 31);
  unsigned char str[270], pMemoText[1024];
  int top, left;


  // Initialize to default values
  oldState = okf;
  OkfSetToDefaults();
  okf.font = oldState.font;
  okf.clip.x1 = 0;
  okf.clip.y1 = 0;
  okf.clip.x2 = 319;
  okf.clip.y2 = 214;
  pFont = OkfGetCurrentFont();
  pShadowE = &okf.shadow.effect;
  pShadowE->intensity = 31;
  top = 5;

  while(GpKeyGet());
  while(TRUE)
  {
    // Screen background
    if (showScreenBackground)
      GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, 320, 215,
                 (unsigned char *) background_bitmap, 0, 0, 320, 240); 
    else
      GpRectFill16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, 215, bgColor);
    GpRectFill16(NULL, &gtSurface[giSurface], 0, 215, GPC_LCD_WIDTH, 1, RGB(0, 0, 0)); // Horizontal line
    GpRectFill16(NULL, &gtSurface[giSurface], 0, 216, GPC_LCD_WIDTH, 24, RGB(31, 31, 31));

    // Sample text
    switch (okf.justify)
    {
    case okfJustifyLeft:  left = 10; break;
    case okfCenter:       left = 160; break;
    case okfJustifyRight: left = 310; break;
    }

    gm_memset(str, 0, sizeof(str));
    gm_memcpy(str, pFont->charSet, pFont->pOkf->nbChars);
    gm_strcat(str, "\n\n"
                   "OKFTESTER EXAMPLE TEXT FOR THE FONT TOOLBOX.\n"
                   "OKFTester example text for the Font Toolbox.");
    OkfSplitLines(str, GPC_LCD_WIDTH - 20, pMemoText);
    OkfPrintAt(left, top, pMemoText);

    // Save current state
    oldState = okf;

    // Current menu option
    OkfSetToDefaults();
    okf.font = 3;
    okf.x = 10; okf.y = 220;

    menuKeys = 0;
    switch (curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000 +
            curOption3 * 100 + curOption4 * 10 + curOption5)
    {
    case 0:
      OkfPrint("Press SELECT for help, START to exit");
      menuKeys = MENUUP | MENUDOWN;
      break;

    case 100000:
      OkfPrint("Current font: ");
      OkfPrintLong(ListGetIndexByHandle(okf.pFonts, oldState.font) + 1, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENUFA | MENUFB;
      break;

    case 200000:
      OkfPrint("Screen background");
      menuKeys = MENUUP | MENUDOWN | MENURIGHT;
      break;
    case 210000:
      OkfPrint("Screen background/Type: ");
      OkfPrint(showScreenBackground ? "Bitmap" : "Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 220000:
      OkfPrint("Screen background/Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 221000:
      OkfPrint("Screen background/Color/Set to");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 221100:
      OkfPrint("Screen background/Color/Set to/Black");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 221200:
      OkfPrint("Screen background/Color/Set to/White");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 221300:
      OkfPrint("Screen background/Color/Set to/Grey");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 221400:
      OkfPrint("Screen background/Color/Set to/Red");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 221500:
      OkfPrint("Screen background/Color/Set to/Green");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 221600:
      OkfPrint("Screen background/Color/Set to/Blue");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 222000:
      OkfPrint("Screen background/Color/Red: ");
      OkfPrintLong(GetRGBRed(bgColor), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 223000:
      OkfPrint("Screen background/Color/Green: ");
      OkfPrintLong(GetRGBGreen(bgColor), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 224000:
      OkfPrint("Screen background/Color/Blue: ");
      OkfPrintLong(GetRGBBlue(bgColor), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;

    case 300000:
      OkfPrint("Font settings");
      menuKeys = MENUUP | MENUDOWN | MENURIGHT;
      break;
    case 310000:
      OkfPrint("Font settings/Opacity: ");
      OkfPrintLong(oldState.opacity, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 320000:
      OkfPrint("Font settings/FX");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 321000:
      OkfPrint("Font settings/FX/Type: ");
      switch (oldState.effect.type)
      {
      case okfNone:          gm_strcpy(str, "None"); break;
      case okfOverlay:       gm_strcpy(str, "Overlay"); break;
      case okfAddColor:      gm_strcpy(str, "AddColor"); break;
      case okfSubtractColor: gm_strcpy(str, "SubtractColor"); break;
      case okfGrayScale:     gm_strcpy(str, "GrayScale"); break;
      }
      OkfPrint(str);
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322000:
      OkfPrint("Font settings/FX/Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 322100:
      OkfPrint("Font settings/FX/Color/Set to");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 322110:
      OkfPrint("Font settings/FX/Color/Set to/Black");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322120:
      OkfPrint("Font settings/FX/Color/Set to/White");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322130:
      OkfPrint("Font settings/FX/Color/Set to/Grey");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322140:
      OkfPrint("Font settings/FX/Color/Set to/Red");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322150:
      OkfPrint("Font settings/FX/Color/Set to/Green");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322160:
      OkfPrint("Font settings/FX/Color/Set to/Blue");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 322200:
      OkfPrint("Font settings/FX/Color/Red: ");
      OkfPrintLong(GetRGBRed(oldState.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 322300:
      OkfPrint("Font settings/FX/Color/Green: ");
      OkfPrintLong(GetRGBGreen(oldState.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 322400:
      OkfPrint("Font settings/FX/Color/Blue: ");
      OkfPrintLong(GetRGBBlue(oldState.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 323000:
      OkfPrint("Font settings/FX/Intensity: ");
      OkfPrintLong(oldState.effect.intensity, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 330000:
      OkfPrint("Font settings/Justification: ");
      gm_strcpy(str, oldState.justify == okfJustifyLeft ? "Left" :
                     oldState.justify == okfCenter ? "Center" : "Right");
      OkfPrint(str);
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 340000:
      OkfPrint("Font settings/Char separation: ");
      OkfPrintLong(oldState.charSeparation, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 350000:
      OkfPrint("Font settings/Line separation: ");
      OkfPrintLong(oldState.lineSeparation, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;

    case 400000:
      OkfPrint("Mask");
      menuKeys = MENUUP | MENUDOWN | MENURIGHT;
      break;
    case 410000:
      OkfPrint("Mask/Active: ");
      OkfPrint(oldState.useMask ? "Yes" : "No");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 420000:
      OkfPrint("Mask/Shadow");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 421000:
      OkfPrint("Mask/Shadow/Opacity: ");
      OkfPrintLong(oldState.shadow.opacity, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 422000:
      OkfPrint("Mask/Shadow/FX");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 422100:
      OkfPrint("Mask/Shadow/FX/Type: ");
      switch (oldState.shadow.effect.type)
      {
      case okfNone:          gm_strcpy(str, "None"); break;
      case okfOverlay:       gm_strcpy(str, "Overlay"); break;
      case okfAddColor:      gm_strcpy(str, "AddColor"); break;
      case okfSubtractColor: gm_strcpy(str, "SubtractColor"); break;
      case okfGrayScale:     gm_strcpy(str, "GrayScale"); break;
      }
      OkfPrint(str);
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422200:
      OkfPrint("Mask/Shadow/FX/Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 422210:
      OkfPrint("Mask/Shadow/FX/Color/Set to");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 422211:
      OkfPrint("Mask/Shadow/FX/Color/Set to/Black");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422212:
      OkfPrint("Mask/Shadow/FX/Color/Set to/White");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422213:
      OkfPrint("Mask/Shadow/FX/Color/Set to/Grey");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422214:
      OkfPrint("Mask/Shadow/FX/Color/Set to/Red");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422215:
      OkfPrint("Mask/Shadow/FX/Color/Set to/Green");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422216:
      OkfPrint("Mask/Shadow/FX/Color/Set to/Blue");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 422220:
      OkfPrint("Mask/Shadow/FX/Color/Red: ");
      OkfPrintLong(GetRGBRed(oldState.shadow.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 422230:
      OkfPrint("Mask/Shadow/FX/Color/Green: ");
      OkfPrintLong(GetRGBGreen(oldState.shadow.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 422240:
      OkfPrint("Mask/Shadow/FX/Color/Blue: ");
      OkfPrintLong(GetRGBBlue(oldState.shadow.effect.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 422300:
      OkfPrint("Mask/Shadow/FX/Intensity: ");
      OkfPrintLong(oldState.shadow.effect.intensity, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 423000:
      if (moveShadow)
      {
        gm_sprintf(str, "X: %ld  Y: %ld",
                   oldState.shadow.x, oldState.shadow.y);
        OkfPrint(str);
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT | MENUFB;
      }
      else
      {
        OkfPrint("Mask/Shadow/Positionning: Start");
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      }
      break;

    case 500000:
      OkfPrint("Background");
      menuKeys = MENUUP | MENUDOWN | MENURIGHT;
      break;
    case 510000:
      OkfPrint("Background/Opacity: ");
      OkfPrintLong(oldState.background.opacity, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 520000:
      OkfPrint("Background/Type: ");
      OkfPrint(oldState.background.onPaint ? "Callback function" : "Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 530000:
      OkfPrint("Background/Color");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 531000:
      OkfPrint("Background/Color/Set to");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT;
      break;
    case 531100:
      OkfPrint("Background/Color/Set to/Black");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 531200:
      OkfPrint("Background/Color/Set to/White");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 531300:
      OkfPrint("Background/Color/Set to/Grey");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 531400:
      OkfPrint("Background/Color/Set to/Red");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 531500:
      OkfPrint("Background/Color/Set to/Green");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 531600:
      OkfPrint("Background/Color/Set to/Blue");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 532000:
      OkfPrint("Background/Color/Red: ");
      OkfPrintLong(GetRGBRed(oldState.background.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 533000:
      OkfPrint("Background/Color/Green: ");
      OkfPrintLong(GetRGBGreen(oldState.background.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 534000:
      OkfPrint("Background/Color/Blue: ");
      OkfPrintLong(GetRGBBlue(oldState.background.color), "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    case 540000:
      if (moveBackground)
      {
        gm_sprintf(str, "X: %ld  Y: %ld",
                   oldState.background.x, oldState.background.y);
        OkfPrint(str);
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT | MENUFB;
      }
      else
      {
        OkfPrint("Background/Positionning: Start");
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      }
      break;
    case 550000:
      if (resizeBackground)
      {
        gm_sprintf(str, "Horiz. indent: %ld  Vert. indent: %ld",
                   oldState.background.horizontalIndent, oldState.background.verticalIndent);
        OkfPrint(str);
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT | MENUFB;
      }
      else
      {
        OkfPrint("Background/Indents: Start");
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      }
      break;

    case 600000:
      OkfPrint("Advanced");
      menuKeys = MENUUP | MENUDOWN | MENURIGHT;
      break;
    case 610000:
      if (clipFont)
      {
        gm_sprintf(str, "From %ld,%ld to %ld,%ld",
                   oldState.clip.x1, oldState.clip.y1, oldState.clip.x2, oldState.clip.y2);
        OkfPrint(str);
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENURIGHT | MENUFB;
      }
      else
      {
        OkfPrint("Advanced/Clipping: Start");
        menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      }
      break;
    case 620000:
      OkfPrint("Advanced/Apply changes");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA;
      break;
    case 630000:
      OkfPrint("Advanced/Alpha LUT levels: ");
      OkfPrintLong(32 >> oldState.alphaLUT.levels, "%ld");
      menuKeys = MENUUP | MENUDOWN | MENULEFT | MENUFA | MENUFB;
      break;
    }

    // Valid menu keys
    str[0] = '\0';
    if (menuKeys & MENUUP)    gm_strcat(str, "\xC7");
    if (menuKeys & MENUDOWN)  gm_strcat(str, "\xC8");
    if (menuKeys & MENULEFT)  gm_strcat(str, "\xC5");
    if (menuKeys & MENURIGHT) gm_strcat(str, "\xC6");
    if (menuKeys & MENUFA)    gm_strcat(str, "\xA1");
    if (menuKeys & MENUFB)    gm_strcat(str, "\xA2");
    okf.font = 5;
    okf.justify = okfJustifyRight;
    OkfPrintAt(310, okf.y - 2, str);

    // Restore state
    okf = oldState;

    SurfaceFlip();

    // Get buttons state
    oldKeys = pressedKeys;
    newKeys = pressedKeys = GpKeyGet();

    // Filter buttons
    if (!moveShadow && !clipFont && (menuKeys & MENURIGHT))
    {
      if (newKeys & GPC_VK_FA)  // If a submenu, A same as RIGHT
      newKeys = (newKeys & ~GPC_VK_FA) | GPC_VK_RIGHT;

      if ((newKeys & GPC_VK_UP) && (oldKeys & GPC_VK_UP))
        newKeys = newKeys & ~GPC_VK_UP;
      if ((newKeys & GPC_VK_DOWN) && (oldKeys & GPC_VK_DOWN))
        newKeys = newKeys & ~GPC_VK_DOWN;
      if ((newKeys & GPC_VK_LEFT) && (oldKeys & GPC_VK_LEFT))
        newKeys = newKeys & ~GPC_VK_LEFT;
      if ((newKeys & GPC_VK_RIGHT) && (oldKeys & GPC_VK_RIGHT))
        newKeys = newKeys & ~GPC_VK_RIGHT;
    }

    // Special key delay for first displacements
    if (!(newKeys & (GPC_VK_UP | GPC_VK_DOWN | GPC_VK_LEFT | GPC_VK_RIGHT | GPC_VK_FA | GPC_VK_FB)))
      keyTick = 0;
    else
      if (keyTick == 0)
        keyTick = GpTickCountGet();
      else
        if (keyTick != -1)
          if (GpTickCountGet() - keyTick < 300)
            newKeys = newKeys & ~(GPC_VK_UP | GPC_VK_DOWN | GPC_VK_LEFT | GPC_VK_RIGHT | GPC_VK_FA | GPC_VK_FB);
          else
            keyTick = -1;

    // Manage buttons
    if (newKeys & GPC_VK_START)
    {
      break;
    }
    if (newKeys & GPC_VK_SELECT)
    {
      Help(curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000 +
           curOption3 * 100 + curOption4 * 10 + curOption5);
    }
    if (newKeys & GPC_VK_FL)
    {
      top--;
    }
    else if (newKeys & GPC_VK_FR)
    {
      top++;
    }
    else if (newKeys & (GPC_VK_FA | GPC_VK_FB))
    {
      switch (curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000 +
              curOption3 * 100 + curOption4 * 10 + curOption5)
      {
      case 100000:
        if (newKeys & GPC_VK_FA)
          if ((pItem = ListGetItemByHandle(okf.pFonts, okf.font)->pNext) != NULL)
            okf.font = pItem->handle;
          else
            okf.font = okf.pFonts->pFirst->handle;
        else
          if ((pItem = ListGetItemByHandle(okf.pFonts, okf.font)->pPrevious) != NULL)
            okf.font = pItem->handle;
          else
            okf.font = okf.pFonts->pLast->handle;
        pFont = OkfGetCurrentFont();
        break;

      case 210000:
        showScreenBackground = !showScreenBackground;
        break;
      case 221100:
        bgColor = RGB(0, 0, 0);
        break;
      case 221200:
        bgColor = RGB(31, 31, 31);
        break;
      case 221300:
        bgColor = RGB(15, 15, 15);
        break;
      case 221400:
        bgColor = RGB(31, 0, 0);
        break;
      case 221500:
        bgColor = RGB(0, 31, 0);
        break;
      case 221600:
        bgColor = RGB(0, 0, 31);
        break;
      case 222000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBRed(bgColor) < 31)
            bgColor = RGB(GetRGBRed(bgColor) + 1, GetRGBGreen(bgColor), GetRGBBlue(bgColor));
          else
            bgColor = RGB(0, GetRGBGreen(bgColor), GetRGBBlue(bgColor));
        else
          if (GetRGBRed(bgColor) > 0)
            bgColor = RGB(GetRGBRed(bgColor) - 1, GetRGBGreen(bgColor), GetRGBBlue(bgColor));
          else
            bgColor = RGB(31, GetRGBGreen(bgColor), GetRGBBlue(bgColor));
        break;
      case 223000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBGreen(bgColor) < 31)
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor) + 1, GetRGBBlue(bgColor));
          else
            bgColor = RGB(GetRGBRed(bgColor), 0, GetRGBBlue(bgColor));
        else
          if (GetRGBGreen(bgColor) > 0)
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor) - 1, GetRGBBlue(bgColor));
          else
            bgColor = RGB(GetRGBRed(bgColor), 31, GetRGBBlue(bgColor));
        break;
      case 224000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBBlue(bgColor) < 31)
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor), GetRGBBlue(bgColor) + 1);
          else
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor), 0);
        else
          if (GetRGBBlue(bgColor) > 0)
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor), GetRGBBlue(bgColor) - 1);
          else
            bgColor = RGB(GetRGBRed(bgColor), GetRGBGreen(bgColor), 31);
        break;

      case 310000:
        if (newKeys & GPC_VK_FA)
          if (okf.opacity < 31)
            okf.opacity++;
          else
            okf.opacity = 0;
        else
          if (okf.opacity > 0)
            okf.opacity--;
          else
            okf.opacity = 31;
        break;
      case 321000:
        if (++okf.effect.type > okfGrayScale)
          okf.effect.type = okfNone;
        break;
      case 322110:
        okf.effect.color = RGB(0, 0, 0);
        break;
      case 322120:
        okf.effect.color = RGB(31, 31, 31);
        break;
      case 322130:
        okf.effect.color = RGB(15, 15, 15);
        break;
      case 322140:
        okf.effect.color = RGB(31, 0, 0);
        break;
      case 322150:
        okf.effect.color = RGB(0, 31, 0);
        break;
      case 322160:
        okf.effect.color = RGB(0, 0, 31);
        break;
      case 322200:
        if (newKeys & GPC_VK_FA)
          if (GetRGBRed(okf.effect.color) < 31)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color) + 1, GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color));
          else
            okf.effect.color = RGB(0, GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color));
        else
          if (GetRGBRed(okf.effect.color) > 0)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color) - 1, GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color));
          else
            okf.effect.color = RGB(31, GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color));
        break;
      case 322300:
        if (newKeys & GPC_VK_FA)
          if (GetRGBGreen(okf.effect.color) < 31)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color) + 1, GetRGBBlue(okf.effect.color));
          else
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), 0, GetRGBBlue(okf.effect.color));
        else
          if (GetRGBGreen(okf.effect.color) > 0)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color) - 1, GetRGBBlue(okf.effect.color));
          else
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), 31, GetRGBBlue(okf.effect.color));
        break;
      case 322400:
        if (newKeys & GPC_VK_FA)
          if (GetRGBBlue(okf.effect.color) < 31)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color) + 1);
          else
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color), 0);
        else
          if (GetRGBBlue(okf.effect.color) > 0)
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color), GetRGBBlue(okf.effect.color) - 1);
          else
            okf.effect.color = RGB(GetRGBRed(okf.effect.color), GetRGBGreen(okf.effect.color), 31);
        break;
      case 323000:
        if (newKeys & GPC_VK_FA)
          if (okf.effect.intensity < 31)
            okf.effect.intensity++;
          else
            okf.effect.intensity = 0;
        else
          if (okf.effect.intensity > 0)
            okf.effect.intensity--;
          else
            okf.effect.intensity = 31;
        break;
      case 330000:
        if (++okf.justify > okfJustifyRight)
          okf.justify = okfJustifyLeft;
        break;
      case 340000:
        if (newKeys & GPC_VK_FA)
        {
          if (okf.charSeparation < 127)
            okf.charSeparation++;
        }
        else
          if (okf.charSeparation > -128)
            okf.charSeparation--;
        break;
      case 350000:
        if (newKeys & GPC_VK_FA)
        {
          if (okf.lineSeparation < 127)
            okf.lineSeparation++;
        }
        else
          if (okf.lineSeparation > -128)
            okf.lineSeparation--;
        break;

      case 410000:
        if (newKeys & GPC_VK_FA)
          okf.useMask = !okf.useMask;
        else
          okf.useMask = TRUE;
        break;
      case 421000:
        if (newKeys & GPC_VK_FA)
          if (okf.shadow.opacity < 31)
            okf.shadow.opacity++;
          else
            okf.shadow.opacity = 0;
        else
          if (okf.shadow.opacity > 0)
            okf.shadow.opacity--;
          else
            okf.shadow.opacity = 31;
        break;
      case 422100:
        if (++okf.shadow.effect.type > okfGrayScale)
          okf.shadow.effect.type = okfNone;
        break;
      case 422211:
        pShadowE->color = RGB(0, 0, 0);
        break;
      case 422212:
        pShadowE->color = RGB(31, 31, 31);
        break;
      case 422213:
        pShadowE->color = RGB(15, 15, 15);
        break;
      case 422214:
        pShadowE->color = RGB(31, 0, 0);
        break;
      case 422215:
        pShadowE->color = RGB(0, 31, 0);
        break;
      case 422216:
        pShadowE->color = RGB(0, 0, 31);
        break;
      case 422220:
        if (newKeys & GPC_VK_FA)
          if (GetRGBRed(pShadowE->color) < 31)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color) + 1, GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color));
          else
            pShadowE->color = RGB(0, GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color));
        else
          if (GetRGBRed(pShadowE->color) > 0)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color) - 1, GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color));
          else
            pShadowE->color = RGB(31, GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color));
        break;
      case 422230:
        if (newKeys & GPC_VK_FA)
          if (GetRGBGreen(pShadowE->color) < 31)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color) + 1, GetRGBBlue(pShadowE->color));
          else
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), 0, GetRGBBlue(pShadowE->color));
        else
          if (GetRGBGreen(pShadowE->color) > 0)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color) - 1, GetRGBBlue(pShadowE->color));
          else
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), 31, GetRGBBlue(pShadowE->color));
        break;
      case 422240:
        if (newKeys & GPC_VK_FA)
          if (GetRGBBlue(pShadowE->color) < 31)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color) + 1);
          else
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color), 0);
        else
          if (GetRGBBlue(pShadowE->color) > 0)
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color), GetRGBBlue(pShadowE->color) - 1);
          else
            pShadowE->color = RGB(GetRGBRed(pShadowE->color), GetRGBGreen(pShadowE->color), 31);
        break;
      case 422300:
        if (newKeys & GPC_VK_FA)
          if (pShadowE->intensity < 31)
            pShadowE->intensity++;
          else
            pShadowE->intensity = 0;
        else
          if (pShadowE->intensity > 0)
            pShadowE->intensity--;
          else
            pShadowE->intensity = 31;
        break;
      case 423000:
        if (newKeys & GPC_VK_FA)
          moveShadow = !moveShadow;
        else
          moveShadow = FALSE;
        break;

      case 510000:
        if (newKeys & GPC_VK_FA)
          if (okf.background.opacity < 31)
            okf.background.opacity++;
          else
            okf.background.opacity = 0;
        else
          if (okf.background.opacity > 0)
            okf.background.opacity--;
          else
            okf.background.opacity = 31;
        break;
      case 520000:
        if (okf.background.onPaint)
          okf.background.onPaint = NULL;
        else
          okf.background.onPaint = &CustomBackground;
        break;
      case 531100:
        okf.background.color = RGB(0, 0, 0);
        break;
      case 531200:
        okf.background.color = RGB(31, 31, 31);
        break;
      case 531300:
        okf.background.color = RGB(15, 15, 15);
        break;
      case 531400:
        okf.background.color = RGB(31, 0, 0);
        break;
      case 531500:
        okf.background.color = RGB(0, 31, 0);
        break;
      case 531600:
        okf.background.color = RGB(0, 0, 31);
        break;
      case 532000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBRed(bgColor) < 31)
            okf.background.color = RGB(GetRGBRed(okf.background.color) + 1, GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color));
          else
            okf.background.color = RGB(0, GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color));
        else
          if (GetRGBRed(okf.background.color) > 0)
            okf.background.color = RGB(GetRGBRed(okf.background.color) - 1, GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color));
          else
            okf.background.color = RGB(31, GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color));
        break;
      case 533000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBGreen(okf.background.color) < 31)
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color) + 1, GetRGBBlue(okf.background.color));
          else
            okf.background.color = RGB(GetRGBRed(okf.background.color), 0, GetRGBBlue(okf.background.color));
        else
          if (GetRGBGreen(okf.background.color) > 0)
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color) - 1, GetRGBBlue(okf.background.color));
          else
            okf.background.color = RGB(GetRGBRed(okf.background.color), 31, GetRGBBlue(okf.background.color));
        break;
      case 534000:
        if (newKeys & GPC_VK_FA)
          if (GetRGBBlue(okf.background.color) < 31)
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color) + 1);
          else
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color), 0);
        else
          if (GetRGBBlue(okf.background.color) > 0)
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color), GetRGBBlue(okf.background.color) - 1);
          else
            okf.background.color = RGB(GetRGBRed(okf.background.color), GetRGBGreen(okf.background.color), 31);
        break;
      case 540000:
        if (newKeys & GPC_VK_FA)
          moveBackground = !moveBackground;
        else
          moveBackground = FALSE;
        break;
      case 550000:
        if (newKeys & GPC_VK_FA)
          resizeBackground = !resizeBackground;
        else
          resizeBackground = FALSE;
        break;

      case 610000:
        if (newKeys & GPC_VK_FA)
          clipFont = !clipFont;
        else
          clipFont = FALSE;
        if (clipFont)
        {
          okf.clip.x1 = 120;
          okf.clip.y1 = 80;
          okf.clip.x2 = okf.clip.x1 + 79;
          okf.clip.y2 = okf.clip.y1 + 54;
        }
        else
        {
          okf.clip.x1 = 0;
          okf.clip.y1 = 0;
          okf.clip.x2 = 319;
          okf.clip.y2 = 214;
        }
        break;

      case 620000:
        if (newKeys & GPC_VK_FA)
        {
          OkfApplyEffect(okf.font, okf.effect.type, okf.effect.color, okf.effect.intensity);
          OkfGetCurrentFont()->pOkf->charSeparation += okf.charSeparation;
          OkfGetCurrentFont()->pOkf->lineSeparation += okf.lineSeparation;

          OkfSetToDefaults();
          menuLevel--;
          curOption1 = 0;
        }
        break;

      case 630000:
        switch ((okf.alphaLUT.levels + 1) % 4)
        {
        case okf4Levels:  OkfInitializeAlphaLUT(okf4Levels,  gtAlpha4);  break;
        case okf8Levels:  OkfInitializeAlphaLUT(okf8Levels,  gtAlpha8);  break;
        case okf16Levels: OkfInitializeAlphaLUT(okf16Levels, gtAlpha16); break;
        case okf32Levels: OkfInitializeAlphaLUT(okf32Levels, gtAlpha32); break;
        }
        break;
      }
    }
    else if (moveShadow)
    {
      if (newKeys & GPC_VK_LEFT)
      {
        if (okf.shadow.x > -32)
          okf.shadow.x--;
      }
      else if (newKeys & GPC_VK_RIGHT)
      {
        if (okf.shadow.x < 32)
          okf.shadow.x++;
      }
      if (newKeys & GPC_VK_UP)
      {
        if (okf.shadow.y > -32)
          okf.shadow.y--;
      }
      else if (newKeys & GPC_VK_DOWN)
      {
        if (okf.shadow.y < 32)
          okf.shadow.y++;
      }
    }
    else if (moveBackground)
    {
      if (newKeys & GPC_VK_LEFT)
      {
        if (okf.background.x > -32)
          okf.background.x--;
      }
      else if (newKeys & GPC_VK_RIGHT)
      {
        if (okf.background.x < 32)
          okf.background.x++;
      }
      if (newKeys & GPC_VK_UP)
      {
        if (okf.background.y > -32)
          okf.background.y--;
      }
      else if (newKeys & GPC_VK_DOWN)
      {
        if (okf.background.y < 32)
          okf.background.y++;
      }
    }
    else if (resizeBackground)
    {
      if (newKeys & GPC_VK_LEFT)
      {
        if (okf.background.horizontalIndent > -32)
          okf.background.horizontalIndent--;
      }
      else if (newKeys & GPC_VK_RIGHT)
      {
        if (okf.background.horizontalIndent < 32)
          okf.background.horizontalIndent++;
      }
      if (newKeys & GPC_VK_UP)
      {
        if (okf.background.verticalIndent > -32)
          okf.background.verticalIndent--;
      }
      else if (newKeys & GPC_VK_DOWN)
      {
        if (okf.background.verticalIndent < 32)
          okf.background.verticalIndent++;
      }
    }
    else if (clipFont)
    {
      if (newKeys & GPC_VK_LEFT)
      {
        if (okf.clip.x1 > 0)
          okf.clip.x1--;
      }
      else if (newKeys & GPC_VK_RIGHT)
      {
        if (okf.clip.x1 + 79 < 319)
          okf.clip.x1++;
      }
      if (newKeys & GPC_VK_UP)
      {
        if (okf.clip.y1 > 0)
          okf.clip.y1--;
      }
      else if (newKeys & GPC_VK_DOWN)
      {
        if (okf.clip.y1 + 54 < 214)
          okf.clip.y1++;
      }
      okf.clip.x2 = okf.clip.x1 + 79;
      okf.clip.y2 = okf.clip.y1 + 54;
    }
    else if (newKeys & (GPC_VK_UP | GPC_VK_DOWN))
    {
      switch (menuLevel)
      {
      case 0: if (newKeys & GPC_VK_DOWN)
              { if (++curOption0 > 6) curOption0 = 1; }
              else
              { if (--curOption0 < 1) curOption0 = 6; }
              break;
      case 1:
        switch (curOption0 * 100000)
        {
        case 200000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption1 > 2) curOption1 = 1; }
                     else
                     { if (--curOption1 < 1) curOption1 = 2; }
                     break;
        case 300000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption1 > 5) curOption1 = 1; }
                     else
                     { if (--curOption1 < 1) curOption1 = 5; }
                     break;
        case 400000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption1 > 2) curOption1 = 1; }
                     else
                     { if (--curOption1 < 1) curOption1 = 2; }
                     break;
        case 500000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption1 > 5) curOption1 = 1; }
                     else
                     { if (--curOption1 < 1) curOption1 = 5; }
                     break;
        case 600000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption1 > 3) curOption1 = 1; }
                     else
                     { if (--curOption1 < 1) curOption1 = 3; }
                     break;
        }
        break;
      case 2:
        switch (curOption0 * 100000 + curOption1 * 10000)
        {
        case 220000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption2 > 4) curOption2 = 1; }
                     else
                     { if (--curOption2 < 1) curOption2 = 4; }
                     break;
        case 320000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption2 > 3) curOption2 = 1; }
                     else
                     { if (--curOption2 < 1) curOption2 = 3; }
                     break;
        case 420000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption2 > 3) curOption2 = 1; }
                     else
                     { if (--curOption2 < 1) curOption2 = 3; }
                     break;
        case 530000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption2 > 4) curOption2 = 1; }
                     else
                     { if (--curOption2 < 1) curOption2 = 4; }
                     break;
        }
        break;
      case 3:
        switch (curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000)
        {
        case 221000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption3 > 6) curOption3 = 1; }
                     else
                     { if (--curOption3 < 1) curOption3 = 6; }
                     break;
        case 322000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption3 > 4) curOption3 = 1; }
                     else
                     { if (--curOption3 < 1) curOption3 = 4; }
                     break;
        case 422000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption3 > 3) curOption3 = 1; }
                     else
                     { if (--curOption3 < 1) curOption3 = 3; }
                     break;
        case 531000: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption3 > 6) curOption3 = 1; }
                     else
                     { if (--curOption3 < 1) curOption3 = 6; }
                     break;
        }
        break;
      case 4:
        switch (curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000 +
                curOption3 * 100)
        {
        case 322100: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption4 > 6) curOption4 = 1; }
                     else
                     { if (--curOption4 < 1) curOption4 = 6; }
                     break;
        case 422200: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption4 > 4) curOption4 = 1; }
                     else
                     { if (--curOption4 < 1) curOption4 = 4; }
                     break;
        }
        break;
      case 5:
        switch (curOption0 * 100000 + curOption1 * 10000 + curOption2 * 1000 +
                curOption3 * 100 + curOption4 * 10)
        {
        case 422210: if (newKeys & GPC_VK_DOWN)
                     { if (++curOption5 > 6) curOption5 = 1; }
                     else
                     { if (--curOption5 < 1) curOption5 = 6; }
                     break;
        }
        break;
      }
    }
    else if (newKeys & (GPC_VK_LEFT | GPC_VK_RIGHT))
    {
      if (newKeys & GPC_VK_LEFT)
      {
        if (menuKeys & MENULEFT)
        {
          menuLevel--;
          if (menuLevel < 1) curOption1 = 0;
          if (menuLevel < 2) curOption2 = 0;
          if (menuLevel < 3) curOption3 = 0;
          if (menuLevel < 4) curOption4 = 0;
          if (menuLevel < 5) curOption5 = 0;
        }
        else
          curOption0 = 0;
      }
      else
        if (menuKeys & MENURIGHT)
        {
          menuLevel++;
          if (menuLevel == 1) curOption1 = 1;
          if (menuLevel == 2) curOption2 = 1;
          if (menuLevel == 3) curOption3 = 1;
          if (menuLevel == 4) curOption4 = 1;
          if (menuLevel == 5) curOption5 = 1;
        }
    }
  }

  while(GpKeyGet());
}
